#include <stdio.h>
#include <stdlib.h>
#include <mem.h>
#include <string.h>

#define BYTE unsigned char
#define TRDSIZE 655360
#define BLOCKSIZE 32768
#define DESCSIZE 16

BYTE trd[TRDSIZE];
BYTE filebuf[TRDSIZE];
BYTE descbuf[DESCSIZE+1]; //for hobeta

const BYTE trdheader[0x20] = {
0x00,0x00,0x01,0x16,0x00,0xf0,0x09,0x10, 0x00,0x00,0x20,0x20,0x20,0x20,0x20,0x20,
0x20,0x20,0x20,0x00,0x00,0x20,0x20,0x20, 0x20,0x20,0x20,0x20,0x20,0x00,0x00,0x00
};

void readtrd(char * trdname)
{
FILE* file;
  file = fopen(trdname, "rb");
  if (file) {
    fread(trd, 1, TRDSIZE, file);
    fclose(file);
  };
}

void writetrd(char * trdname)
{
FILE* file;
  file = fopen(trdname, "wb");
  if (file) {
    fwrite(trd, 1, TRDSIZE, file);
    fclose(file);
  };
}

void genname(char * filename)
{
int i;
char c;
  i = 0;
  while (i<9) {
    descbuf[i] = ' ';
    i++;
  };
  i = 0;
  while (i<9) {
    c = filename[i];
    if (c=='\0') break;
    if (c=='.') {
      i++;
      c = filename[i];
      if (c=='\0') break;
      if (c=='$') i++;
      c = filename[i];
      if (c=='\0') break;
      descbuf[8] = c;
      break;
    };
    descbuf[i] = c;
    i++;
  };
}

long finddesc(int namelen, int ignorereg) //return descriptor address or -1 if not found
{
long descaddr;
long result;
int i;
BYTE err;
  result = -1;
  descaddr = 0;
  while (trd[descaddr] != 0) {
    err = 0;
    i = 0;
    while (i < namelen) { //name, ext, block
      if (ignorereg) {
        err = err | ( ((BYTE)trd[descaddr+i]|(BYTE)0x20) ^ ((BYTE)descbuf[i]|(BYTE)0x20) );
      }else {
        err = err | (trd[descaddr+i]^descbuf[i]);
      };
      i++;
    };
    if (err == 0) {
      result = descaddr;
      break;
    };
    descaddr = descaddr + DESCSIZE;
  };
  return result;
}

void extractfile(char * trdname, char * filename)
{
FILE* fout;
int descaddr;
int addr; //in sectors
BYTE block;
int wrsize;
  readtrd(trdname);
  fout = fopen(filename, "wb");
  if (fout) {
    genname(filename);

    block = 0;
    do {
      descbuf[9] = block;
      descaddr = finddesc(10, 0); //name, ext, block
      wrsize = 0;
      if (descaddr != -1) {
        wrsize = trd[descaddr+11] + trd[descaddr+12]*256;
        addr = trd[descaddr+14] + trd[descaddr+15]*16;
      };
      if (wrsize != 0) {
        fwrite(trd+(addr<<8), 1, wrsize, fout);
      };
      block++;
    }while (wrsize != 0);

    fclose(fout);
  };
}

void extractbinary(char * trdname, char * filename)
{
FILE* fout;
int descaddr;
int addr; //in sectors
int wrsize;
  readtrd(trdname);
  fout = fopen(filename, "wb");
  if (fout) {
    genname(filename);

    descaddr = finddesc(9, 0); //name, ext, block
    wrsize = 0;
    if (descaddr != -1) {
      wrsize = trd[descaddr+11] + trd[descaddr+12]*256;
      addr = trd[descaddr+14] + trd[descaddr+15]*16;
    };
    if (wrsize != 0) {
      fwrite(trd+(addr<<8), 1, wrsize, fout);
    };

    fclose(fout);
  };
}

void extracthobeta(char * trdname, char * filename)
{
FILE* fout;
int descaddr;
int addr; //in sectors
int wrsize;
unsigned int checksum;
int i;
  readtrd(trdname);
  fout = fopen(filename, "wb");
  if (fout) {
    genname(filename);
/**
p 13    p .    
 p; ..  p 256,  p  ,  p -
 p.     - p . 
 p - p  p 15 ,   
257  p ___0__14 .. 105.
 pp  Z80 Asm:
;   de = p 
ld hl,0
ld b,15
m1: ld a,(de)
add a,l
ld l,a
jr nc,m2
inc h
m2: inc de
djnz m1
add a,h
ld h,a
ld c,105
add hl,bc ; hl = Hobeta sum
*/
    descaddr = finddesc(9, 1); //name, ext
    if (descaddr != -1) {
      memcpy(descbuf, trd+descaddr, DESCSIZE); //descriptor
      wrsize = trd[descaddr+13]*256;
      addr = trd[descaddr+14] + trd[descaddr+15]*16;
      descbuf[14] = descbuf[13];
      descbuf[13] = 0;
      for (i=0; i<=14; checksum = checksum + (descbuf[i] * 257) + i, i++);
      descbuf[15] = (BYTE)checksum;
      descbuf[16] = (BYTE)(checksum>>8);
      fwrite(descbuf, 1, 17, fout);
      fwrite(trd+(addr<<8), 1, wrsize, fout);
    };

    fclose(fout);
  };
  //writetrd(trdname);
}

void addfile(char * trdname, char * filename, int iscode, int startaddr)
{
FILE* fin;
long descaddr;
long addr; //in sectors
long size;
long wrsize;
long secsize;
long index;
long free;
BYTE block;
long maxsize;
  readtrd(trdname);

  fin = fopen(filename, "rb");
  if (fin) {
    size = fread(filebuf, 1, TRDSIZE, fin);

    genname(filename);
    if (iscode) {
      descbuf[8] = 'C';
      maxsize = 65280;
    }else {
      maxsize = BLOCKSIZE;
    };

    index = 0;
    block = (BYTE)startaddr;
    while (size > 0) {

      if (size > maxsize) {
        wrsize = maxsize;
      }else {
        wrsize = size;
      };
      descbuf[9] = block;
      descbuf[10] = (BYTE)(startaddr>>8);
      descbuf[11] = (BYTE)wrsize;
      descbuf[12] = (BYTE)(wrsize>>8);
      secsize = (wrsize+255)>>8;
      descbuf[13] = (BYTE)secsize;

      addr = trd[0x08e1] + (((long)trd[0x08e2])<<4); //first free sector, track
      memcpy(trd+(addr<<8), filebuf+index, wrsize); //data
      descbuf[14] = (BYTE)(addr&0x0f); //sector
      descbuf[15] = (BYTE)(addr>>4); //track
      addr = addr + secsize;
      trd[0x08e1] = (BYTE)(addr&0x0f); //first free sector
      trd[0x08e2] = (BYTE)(addr>>4); //first free track

      descaddr = ((long)trd[0x08e4])<<4; //files
      trd[0x08e4]++; //files
      free = trd[0x08e5] + 256*trd[0x08e6]; //free
      free = free - secsize;
      trd[0x08e5] = (BYTE)free;
      trd[0x08e6] = (BYTE)(free>>8);

      memcpy(trd+descaddr, descbuf, DESCSIZE); //descriptor

      size = size - maxsize;
      index = index + maxsize;
      block++;
    };
    fclose(fin);
  };
  writetrd(trdname);
}

void addhobeta(char * trdname, char * filename)
{
FILE* fin;
long descaddr;
long addr; //in sectors
long size;
long wrsize;
long secsize;
long index;
long free;
int i;
  readtrd(trdname);

  fin = fopen(filename, "rb");
  if (fin) {
    size = fread(filebuf, 1, TRDSIZE, fin);

    wrsize = size - 17;
    index = 17;

    i = 0;
    while (i<13) {
      descbuf[i] = filebuf[i];
      i++;
    };
    secsize = filebuf[14];
    descbuf[13] = (BYTE)secsize;

    addr = trd[0x08e1] + (((long)trd[0x08e2])<<4); //first free sector, track
    memcpy(trd+(addr<<8), filebuf+index, wrsize); //data
    descbuf[14] = (BYTE)(addr&0x0f); //sector
    descbuf[15] = (BYTE)(addr>>4); //track
    addr = addr + secsize;
    trd[0x08e1] = (BYTE)(addr&0x0f); //first free sector
    trd[0x08e2] = (BYTE)(addr>>4); //first free track

    descaddr = ((long)trd[0x08e4])<<4; //files
    trd[0x08e4]++; //files
    free = trd[0x08e5] + 256*trd[0x08e6]; //free
    free = free - secsize;
    trd[0x08e5] = (BYTE)free;
    trd[0x08e6] = (BYTE)(free>>8);

    memcpy(trd+descaddr, descbuf, DESCSIZE); //descriptor

    fclose(fin);
  };
  writetrd(trdname);
}

void newtrd(char * trdname)
{
  memset(trd, 0x00, TRDSIZE);
  memcpy(trd+0x08e0, trdheader, 0x0020);
  writetrd(trdname);
}

int main(int argc,char* argv[])
{
int i;
int startaddr;
char *trdname;
char *filename;
  startaddr = 24576;
  trdname = "test.trd";

  if (argc<2) {
    printf(
      "NedoTRD\n"
      "add partitioned file:\n"
      "\tnedotrd.exe trdname.trd -a file.f\n"
      "add code file with default start address (24576):\n"
      "\tnedotrd.exe trdname.trd -ac file.c\n"
      "add code file with start address 32768:\n"
      "\tnedotrd.exe trdname.trd -s 32768 -ac file.c\n"
      "add hobeta file:\n"
      "\tnedotrd.exe trdname.trd -ah file.$b\n"
      "extract partitioned file:\n"
      "\tnedotrd.exe trdname.trd -e file.f\n"
      "extract binary file:\n"
      "\tnedotrd.exe trdname.trd -eb file.c\n"
      "extract hobeta file:\n"
      "\tnedotrd.exe trdname.trd -eh file.$b\n"
      "new trd file:\n"
      "\tnedotrd.exe trdname.trd -n\n"
    );
  };

  for (i=1; i<argc; i++) {
    if       (!strcmp(argv[i],"-a")) { //add
      i++;
      filename = argv[i];
      addfile(trdname, filename, 0, startaddr);
    }else if (!strcmp(argv[i],"-ac")) { //add code
      i++;
      filename = argv[i];
      addfile(trdname, filename, 1, startaddr);
    }else if (!strcmp(argv[i],"-ah")) { //add hobeta
      i++;
      filename = argv[i];
      addhobeta(trdname, filename);
    }else if (!strcmp(argv[i],"-e")) { //extract psrtitioned
      i++;
      filename = argv[i];
      extractfile(trdname, filename);
    }else if (!strcmp(argv[i],"-eb")) { //extract binary
      i++;
      filename = argv[i];
      extractbinary(trdname, filename);
    }else if (!strcmp(argv[i],"-eh")) { //extract hobeta
      i++;
      filename = argv[i];
      extracthobeta(trdname, filename);
    }else if (!strcmp(argv[i],"-n")) { //new trd
      newtrd(trdname);
    }else if (!strcmp(argv[i],"-s")) { //use start address
      i++;
      filename = argv[i];
      startaddr = atoi(filename);
    }else {
      trdname = argv[i];
    };
  }
  return 0;
}